# this was originally used to do the initial conversion of all but
# about 40 lines of the Basic used by Mazogs.


# basconv.awk - designed to do an initial pass at converting
# ZX81 Basic into Z80 assembly. Won't manage anything complicated
# but should cut *some* of the drudgery.
#
# this was written to be used for Mazogs, so many places
# make assumptions based on that - don't use it for anything
# else without checking it first.
#
# notable examples:
# - nested FORs will break, it only copes with one level of them.
# - it assumes that NEXT matches the previous FOR it saw
#   (a similar but not necessarily identical problem)


BEGIN \
{
# for mazogs-specific fixes
mazogs=1

if(mazogs)
  {
  # loosely based on the ord code from the gawk manual
  for(i=32;i<=126;i++)
    {
    j=i
    # all lowercase
    if(j>=65 && j<=90) { j+=32 }
    t=sprintf("%c",i)
    _ord_[t]=j
    }
  }
}


END \
{
print ""
print ""
print ";lines not handled: " nothandled " out of " NR
}


{
num=$1
keyword=$2
$1=""
numlabel="l" num
print ""
print numlabel ":\t\t;" $0
}


# special case for mazogs
mazogs && num>=9000 { next }


keyword=="REM" { next }

keyword=="GOTO" { print "jp l" $3 ; next }

keyword=="GOSUB" { print "call l" $3 ; next }

keyword=="RETURN" { print "ret"; next }

keyword=="POKE" \
  {
  sav=$0
  sub(/,/," ")
  if($2 ~ /^[0-9]*$/ && $3 ~ /^[0-9]*$/)
    {
    if($3==0)
      { print "xor a" }
    else
      { print "ld a," $3 }
    print "ld (" $2 "),a"
    next
    }
  $0=sav
  }

keyword=="PRINT" \
  {
  sav=$0
  sub(/,/," ")
  sub(/;/," ")
  
  # kludge for mazogs...
  if(mazogs && $2=="AT" && $4=="C") { $4="2" }
  
  if($2=="AT" && $3 ~ /^[0-9]*$/ && $4 ~ /^[0-9]*$/ && !/";/ && !/;"/)
    {
    # XXX ilprint should copy c unmodified to lasty
    print "ld c," $3
    print "ld b," $4
    print "call ilprint"
    $1=""
    $2=""
    $3=""
    $4=""
    sub(/"/,"'")
    sub(/"$/,"'")
    gsub(/""/,"\"")
    print "defb " $0 ",0"
    next
    }
  $0=sav
  }

keyword=="FOR" && !/STEP/ \
  {
  sub(/[A-Z]=/,"")
  if($2 ~ /^[0-9]*$/ && $3=="TO" && $4 ~ /^[0-9]*$/)
    {
    print "ld de," $2
    print numlabel "for:"
    print "push de"
    oldlabel=numlabel
    oldmax=$4
    next
    }
  }

keyword=="NEXT" \
  {
  print "pop de"
  print "inc de"
  print "ld hl," oldmax "+1"
  print "and a"
  print "sbc hl,de"
  print "jp nz," oldlabel "for"
  next
  }

/^ *RAND$/ { print "call rand"; next }

/^ *RAND USR [0-9]*$/ { print "call " $4 ; next }

# "let x" variant is for mazogs
mazogs && /LET X=USR [0-9]*$/ { print "call " $4 ; print "ld (var_X),bc"; next }

# these can be ignored in mazogs
mazogs && (keyword=="SLOW" || /LET C=2$/) \
  {
  print ";can be ignored"
  next
  }

# special shortcut case for mazogs
mazogs && num<200 && keyword=="LET" \
  {
  sub(/=/," ")
  print "l" $2 " equ l" $3
  next
  }

# use loop var directly on line 210
mazogs && num==210 \
  {
  print "xor a"
  print "ld (de),a"
  next
  }

mazogs && /^ LET AD=[0-9]*$/ \
  {
  sub(/=/," ")
  print "ld hl," $3
  print "ld (var_AD),hl"
  next
  }

mazogs && /^ LET [A-Z]+=[A-Z]+$/ \
  {
  sub(/=/," ")
  print "ld hl,(var_" $3 ")"
  print "ld (var_" $2 "),hl"
  next
  }

mazogs && /^ LET [A-Z]+=[A-Z]+\+[0-9]+$/ \
  {
  sub(/=/," ")
  sub(/\+/," ")
  print "ld hl,(var_" $3 ")"
  print "ld de," $4
  print "add hl,de"
  print "ld (var_" $2 "),hl"
  next
  }

mazogs && /^ POKE [A-Z]+,[0-9]+$/ \
  {
  sub(/,/," ")
  print "ld hl,(var_" $2 ")"
  print "ld (hl)," $3
  next
  }


mazogs && /^ POKE [A-Z]+[\+-][0-9]+,[0-9]+$/ \
  {
  minus=0
  if(/-/) { minus=1 }
  sub(/,/," ")
  sub(/[\+-]/," ")
  print "ld hl,(var_" $2 ")"
  if(minus)
    {print "ld de,-" $3}
  else
    {print "ld de," $3}
  print "add hl,de"
  print "ld (hl)," $4
  next
  }

mazogs && /^ IF PEEK [0-9]+ THEN GOTO [0-9]+$/ \
  {
  print "ld a,(" $4 ")"
  print "and a"
  print "jp nz,l" $7
  next
  }

mazogs && /^ LET .\$=INKEY\$$/ \
  {
  sub(/\$/," ")
  
  # XXX note that this should be a speccy inkey, getting the result in ASCII
  print "call inkey"
  print "ld (last_" $2 "str),a"
  next
  }

mazogs && /^ IF .\$="." THEN [GR][OU][TN][O ]/ \
  {
  sub(/\$/," ")
  sub(/"/," ")
  sub(/"/,"")
  print "ld a,(last_" $2 "str)"
  print "cp " _ord_[$4]
  print "jp z,l" $7
  next
  }

mazogs && num==1020 \
  {
  print "ld hl,(var_X)"
  print "ld (16525),hl"
  next
  }

mazogs && (num==1022 || num==6020) \
  {
  print ";can be ignored"
  next
  }

mazogs && num==6010 \
  {
  print "ld de,(var_N)"
  print "ld hl,(var_AD)"
  print "ld (hl),e"
  print "inc hl"
  print "ld (hl),d"
  next
  }

mazogs && (/^ LET [A-Z]+=PEEK [0-9]+\+PEEK [0-9]+\*256$/ ||
           /^ LET [A-Z]+=PEEK [0-9]+\+256\*PEEK [0-9]+$/) \
  {
  sub(/=/," ")
  sub(/\+/," ")
  print "ld hl,(" $4 ")"
  print "ld (var_" $2 "),hl"
  next
  }

mazogs && /^ IF INKEY\$ ="" THEN GOTO [0-9]+$/ \
  {
  # XXX assumes it returns zero for no key
  print "call inkey"
  print "and a"
  print "jp z,l" $7
  next
  }

mazogs && /^ IF INKEY\$ <>"" THEN GOTO [0-9]+$/ \
  {
  # XXX assumes it returns zero for no key
  print "call inkey"
  print "and a"
  print "jp nz,l" $7
  next
  }

mazogs && /^ PRINT$/ \
  {
  print "ld hl,lasty"
  print "inc (hl)"
  next
  }


{
print "XXX"
nothandled++
}
